function [fhat,xi] = FourierTransform(f,a)
% INPUTS:
% f is a vector of size N=2^k, representing samples of a function over the
% grid x_j = a/N*(j-N/2), j=0,1,... N-1. i.e. from interval [a/2,a/2[
% OUTPUTS:
% fhat is the fourier transform of f, sampled on grid xi.

N=length(f);
plusminusones=mod(2:2:2*N,4)-1; %Vector [1,-1,1,-1,1,...-1]

fhat=a/N/sqrt(2*pi)*exp(-1i*pi*N/2)*plusminusones.*fft(f.*plusminusones);

xi = 2*pi/a*((0:N-1) - N/2);