% 1d Random Medium Generator

% Medium parameters
    lengthMedium=100000;
    N=2^18;
    h=lengthMedium/N;
    x=h*( (0:N-1)-N/2); % spatial grid
% Desired correlation function
    corr_length=10;
    % Gaussian correlation
        %C=@(x)exp(-x.*x/2/corr_length^2);
    % sinc function
        %C=@(x) sinc(2*pi*x/corr_length);
    % Piramid-shaped correlation
        %C=@(x) 10*heaviside(2*corr_length-abs(x)).*(1-abs(x)/2/corr_length);
    % Cosine
        %C=@(x) cos(x*pi*3/4/corr_length).*heaviside(2*corr_length - abs(x));
    % Smoothed Cosine
        %C=@(x) cos(x*pi*3/4/corr_length).*(1 - abs(x)/2/corr_length).*heaviside(2*corr_length - abs(x));
    % High-low Step
        C=@(x) 1*heaviside(1.5*corr_length-abs(x)) - 0.25*heaviside(2*corr_length - abs(x));
        
% generating Gaussian white noise, the coefficient is as the integration
% multiplies by h.
    n=randn(1,N)*sqrt(1/h);

% Taking the Fourier transforms
    Fourier_n=FourierTransform(n,lengthMedium);
    Fourier_C=FourierTransform(C(x),lengthMedium);
    
% Computing the random medium inverting the Fourier transform
    RandomMedium=sqrt(2*pi)*FourierTransform(sqrt(abs(Fourier_C))/sqrt(sqrt(2*pi)).*Fourier_n,2*pi*N/lengthMedium);

%% Observing random medium statistics

figure
subplot(2,2,1)
obsPoints=N;
plot(x(1:obsPoints),real(RandomMedium(1:obsPoints)))
hold on
plot(x(1:obsPoints),imag(RandomMedium(1:obsPoints)),'r')
hold off
title('Full random medium')

gca=subplot(2,2,2);
% To see the local shape of the random medium, equal to 50 correlation
% lengths
    if lengthMedium>25*corr_length
        aux=find(x-x(1)>25*corr_length);
        obsPoints=aux(1);
    else
        obsPoints=N;
    end
plot(x(1:obsPoints),real(RandomMedium(1:obsPoints)))
hold on
plot(x(1:obsPoints),imag(RandomMedium(1:obsPoints)),'r')
hold off
set(gca,'XLim',[x(1),x(obsPoints)])
title('Observing first 50 correlation lengths')

subplot(2,2,3)
% Distribution of the obtained random medium
    [hh,hhh]=hist(real(RandomMedium),50);
    bar(hhh,hh/sum(hh*(hhh(2)-hhh(1))))
    hold on
% Expected distribution 
    sigma2=C(0);
    disty=@(x)(exp(-x.*x/2/sigma2)/sqrt(2*pi*sigma2));
    plot(hhh,disty(hhh),'r');
    legend('Empirical dist.', 'Theoretical dist.');
    title('Empirical distribution v/s theoretical one');
    
gca=subplot(2,2,4);
% Empirical covariance of the random medium

    % We want to see only close to the 0 for the covariance.
    aux=find(x-x(1)>3*corr_length);
    obsPoints=aux(1);
    if (obsPoints<20) 
        disp('h too small to see properly the covariance, augment N');
    end
    empCov=zeros(obsPoints-1,1);
    waity=waitbar(1/obsPoints,'covariansing');
    for i=1:obsPoints-1
    waitbar(i/obsPoints,waity);
    empCov(i)=sum(RandomMedium(1:end-i).*RandomMedium(1+i:end))/(length(RandomMedium)-i);
    end
    close(waity)
    plot(x(2:obsPoints)-x(1),real(empCov))
    hold on
    % Plotting expected covariance
    plot(x(2:obsPoints)-x(1), C(x(2:obsPoints)-x(1)),'r')
    hold off
    legend('Empirical cov','Theoretical cov');
    title('Empirical covariance v/s theoretical one');
    set(gca,'XLim',[x(2)-x(1),x(obsPoints)-x(1)])
%% 2D Random Medium.

if 0 %to activate or deactivate this section
    
% Medium parameters (For a square, and we treat each variable separatedly)
    lengthMedium=100;
    N2=2^22;
    N=sqrt(N2);
    h=lengthMedium/N;
    x=h*( (0:N-1)-N/2); % spatial grid
    [X,Y]=meshgrid(x,x);
% Desired correlation function
    corr_length=1;
    % Gaussian correlation
        G=@(x)exp(-x.*x/2/corr_length^2);
    % sinc function
        %G=@(x) sinc(2*pi*x/(10*corr_length));
    % Cone-shaped correlation
        %G=@(x) 10*heaviside(2*corr_length-abs(x)).*(1-abs(x)/2/corr_length);
    % Cosine
        %G=@(x) cos(x*pi*3/4/corr_length).*heaviside(2*corr_length - abs(x));
    % Smoothed Cosine
        %G=@(x) cos(x*pi*3/4/corr_length).*(1 - abs(x)/2/corr_length).*heaviside(2*corr_length - abs(x));
    % High-low Step
        %G=@(x) 1*heaviside(1.5*corr_length-abs(x)) - 0.25*heaviside(2*corr_length - abs(x));
        
    C=@(x,y) G(sqrt(x.^2+y.^2));
% generating Gaussian white noise.
    n=randn(N,N)*sqrt(N/lengthMedium)^2;
    
% Taking the Fourier transforms
    Fourier_n=zeros(N,N);
    for i=1:N
       [Fourier_n(:,i),xi]=FourierTransform(n(:,i)',lengthMedium);
    end
    for i=1:N
        Fourier_n(i,:)=FourierTransform(Fourier_n(i,:),lengthMedium);
    end

    Fourier_C=zeros(N,N);
    for i=1:N
        [Fourier_C(:,i),xi]=FourierTransform(C(X(:,i),Y(:,i))',lengthMedium);
    end
    for i=1:N
       Fourier_C(i,:)=FourierTransform(Fourier_C(i,:),lengthMedium);
    end
    
% Computing the random medium inverting the Fourier transform
    
    RandomMedium=sqrt(abs(Fourier_C))/sqrt(2*pi).*Fourier_n;
    for i=1:N
       RandomMedium(:,i)=sqrt(2*pi)*FourierTransform(RandomMedium(:,i)',2*pi*N/lengthMedium);
    end
    for i=1:N
       RandomMedium(i,:)=sqrt(2*pi)*FourierTransform(RandomMedium(i,:),2*pi*N/lengthMedium);
    end
    
 % Plotting
figure
 subplot(1,3,1) 
 % total random medium
    imagesc(x,x,real(RandomMedium))
    title('whole random medium')
 
 
 subplot(1,3,2)
 % zoom to random medium
    if lengthMedium>25*corr_length
        aux=find(x-x(1)>25*corr_length);
        obsPoints=aux(1);
    else
        obsPoints=N;
    end
    imagesc(x(1:obsPoints),x(1:obsPoints),real(RandomMedium(1:obsPoints, 1:obsPoints)))
    title('Closer look')
    
 subplot(1,3,3)
 % Pointwise histogram
% Distribution of the obtained random medium
    [hh,hhh]=hist(real(RandomMedium),50);
    bar(hhh,hh/sum(hh*(hhh(2)-hhh(1))))
    hold on
% Expected distribution 
    sigma2=C(0,0);
    disty=@(x)(exp(-x.*x/2/sigma2)/sqrt(2*pi*sigma2));
    plot(hhh,disty(hhh),'r');
    legend('Empirical dist.', 'Theoretical dist.');
    title('Empirical distribution v/s theoretical one');
end